/****************************************************************
 * Licensed to the Apache Software Foundation (ASF) under one   *
 * or more contributor license agreements.  See the NOTICE file *
 * distributed with this work for additional information        *
 * regarding copyright ownership.  The ASF licenses this file   *
 * to you under the Apache License, Version 2.0 (the            *
 * "License"); you may not use this file except in compliance   *
 * with the License.  You may obtain a copy of the License at   *
 *                                                              *
 *   http://www.apache.org/licenses/LICENSE-2.0                 *
 *                                                              *
 * Unless required by applicable law or agreed to in writing,   *
 * software distributed under the License is distributed on an  *
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY       *
 * KIND, either express or implied.  See the License for the    *
 * specific language governing permissions and limitations      *
 * under the License.                                           *
 ****************************************************************/
package org.apache.james.vut.file;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.HierarchicalConfiguration;
import org.apache.james.vut.lib.AbstractVirtualUserTable;
import org.apache.james.vut.lib.VirtualUserTableUtil;

/**
 * Class responsible to implement the Virtual User Table in XML disk file.
 */
public class XMLVirtualUserTable extends AbstractVirtualUserTable {
    
    /**
     * Holds the configured mappings
     */
    private Map<String,String> mappings;
    
    /**
     * @see org.apache.james.vut.lib.AbstractVirtualUserTable#doConfigure(org.apache.commons.configuration.HierarchicalConfiguration)
     */
    @SuppressWarnings("unchecked")
    protected void doConfigure(HierarchicalConfiguration arg0) throws ConfigurationException {

        List<String> mapConf = arg0.getList("mapping");
    
        mappings = new HashMap<String,String>();
        
        if (mapConf != null && mapConf.size() > 0) {
            for (int i = 0; i < mapConf.size(); i ++) {       
                mappings.putAll(VirtualUserTableUtil.getXMLMappings(mapConf.get(i)));
            }
        } else {
            throw new ConfigurationException("No mapping configured");
        }
        
    }
    
    /**
     * Not implemented
     */
    protected boolean addMappingInternal(String user, String domain, String mapping) {
        // Not supported
        return false;
    }

    /**
     * @see org.apache.james.vut.lib.AbstractVirtualUserTable#mapAddressInternal(java.lang.String, java.lang.String)
     */
    protected String mapAddressInternal(String user, String domain) {
        if (mappings == null) {
            return null;
        } else {
            return VirtualUserTableUtil.getTargetString(user, domain, mappings);
        }
    }

    /**
     * Not implemented
     */
    protected boolean removeMappingInternal(String user, String domain, String mapping) {
        // Not supported
        return false;
    }

    /**
     * @see org.apache.james.vut.lib.AbstractVirtualUserTable#getUserDomainMappingsInternal(java.lang.String, java.lang.String)
     */
    protected Collection<String> getUserDomainMappingsInternal(String user, String domain) {
        if (mappings == null) {
            return null;
        } else {
            String maps = mappings.get(user + "@" + domain);
            if (maps != null) {
                return VirtualUserTableUtil.mappingToCollection(maps);
            } else {
                return null;
            }
        }
    }

    /**
     * @see org.apache.james.vut.lib.AbstractVirtualUserTable#getAllMappingsInternal()
     */
    protected Map<String,Collection<String>> getAllMappingsInternal() {
        if ( mappings != null && mappings.size() > 0) {
            Map<String,Collection<String>> mappingsNew = new HashMap<String,Collection<String>>();
            Iterator<String> maps = mappings.keySet().iterator();
            while (maps.hasNext()) {
                String key = maps.next();
                mappingsNew.put(key, VirtualUserTableUtil.mappingToCollection(mappings.get(key).toString()));
            }
            return mappingsNew;
        } else {
            return null;
        }
    }
    
}
